package commands

import (
	"image"
	"os"
	"path/filepath"
	"strings"

	"fyne.io/fyne/cmd/fyne/internal/templates"
	"fyne.io/fyne/cmd/fyne/internal/util"

	"github.com/jackmordaunt/icns"
	"github.com/pkg/errors"
)

type darwinData struct {
	Name, ExeName string
	AppID         string
	Version       string
	Build         int
	Category      string
}

func (p *packager) packageDarwin() error {
	appDir := util.EnsureSubDir(p.dir, p.name+".app")
	exeName := filepath.Base(p.exe)

	contentsDir := util.EnsureSubDir(appDir, "Contents")
	info := filepath.Join(contentsDir, "Info.plist")
	infoFile, _ := os.Create(info)

	tplData := darwinData{Name: p.name, ExeName: exeName, AppID: p.appID, Version: p.appVersion, Build: p.appBuild,
		Category: strings.ToLower(p.category)}
	err := templates.InfoPlistDarwin.Execute(infoFile, tplData)
	if err != nil {
		return errors.Wrap(err, "Failed to write plist template")
	}

	macOSDir := util.EnsureSubDir(contentsDir, "MacOS")
	binName := filepath.Join(macOSDir, exeName)
	err = util.CopyExeFile(p.exe, binName)
	if err != nil {
		return errors.Wrap(err, "Failed to copy exe file")
	}

	resDir := util.EnsureSubDir(contentsDir, "Resources")
	icnsPath := filepath.Join(resDir, "icon.icns")

	img, err := os.Open(p.icon)
	if err != nil {
		return errors.Wrapf(err, "Failed to open source image \"%s\"", p.icon)
	}
	defer img.Close()
	srcImg, _, err := image.Decode(img)
	if err != nil {
		return errors.Wrapf(err, "Failed to decode source image")
	}
	dest, err := os.Create(icnsPath)
	if err != nil {
		return errors.Wrap(err, "Failed to open destination file")
	}
	defer dest.Close()
	if err := icns.Encode(dest, srcImg); err != nil {
		return errors.Wrap(err, "Failed to encode icns")
	}

	return nil
}
